@@ -1,5 +1,8 @@
-from zope.interface import implements
+# -*- coding: utf-8 -*-
+from zope.interface import implementer_only
+from zope.interface import implementer
 from zope.component import getMultiAdapter, getUtility
+from zope.publisher.interfaces.browser import IBrowserView
 
 from AccessControl import Unauthorized
 from Acquisition import aq_inner
@@ -19,8 +22,10 @@ from plone.portlets.constants import CONTEXT_CATEGORY
 
 from plone.app.portlets.storage import PortletAssignmentMapping
 from plone.app.portlets.storage import UserPortletAssignmentMapping
+from plone.app.portlets.storage import GroupDashboardPortletAssignmentMapping
 
 from plone.app.portlets.interfaces import IPortletPermissionChecker
+from plone.app.portlets.interfaces import ITopbarManagePortlets
 
 from plone.app.portlets.browser.interfaces import IManagePortletsView
 from plone.app.portlets.browser.interfaces import IManageContextualPortletsView
@@ -30,9 +35,11 @@ from plone.app.portlets.browser.interfaces import IManageContentTypePortletsView
 
 from plone.app.portlets import utils
 from plone.memoize.view import memoize
+from plone.protect.authenticator import createToken
 
+
+@implementer(IManageContextualPortletsView)
 class ManageContextualPortlets(BrowserView):
-    implements(IManageContextualPortletsView)
 
     def __init__(self, context, request):
         super(ManageContextualPortlets, self).__init__(context, request)
@@ -69,10 +76,17 @@ class ManageContextualPortlets(BrowserView):
         return (left_slots or right_slots)
 
     # view @@set-portlet-blacklist-status
-    def set_blacklist_status(self, manager, group_status, content_type_status, context_status):
+    def set_blacklist_status(self, manager, group_status, content_type_status,
+                             context_status):
+        authenticator = getMultiAdapter((self.context, self.request),
+                                        name=u"authenticator")
+        if not authenticator.verify():
+            raise Unauthorized
         portletManager = getUtility(IPortletManager, name=manager)
-        assignable = getMultiAdapter((self.context, portletManager,), ILocalPortletAssignmentManager)
-        assignments = getMultiAdapter((self.context, portletManager), IPortletAssignmentMapping)
+        assignable = getMultiAdapter((self.context, portletManager),
+                                     ILocalPortletAssignmentManager)
+        assignments = getMultiAdapter((self.context, portletManager),
+                                      IPortletAssignmentMapping)
 
         IPortletPermissionChecker(assignments.__of__(aq_inner(self.context)))()
 
@@ -85,7 +99,8 @@ class ManageContextualPortlets(BrowserView):
                 return False
 
         assignable.setBlacklistStatus(GROUP_CATEGORY, int2status(group_status))
-        assignable.setBlacklistStatus(CONTENT_TYPE_CATEGORY, int2status(content_type_status))
+        assignable.setBlacklistStatus(CONTENT_TYPE_CATEGORY,
+                                      int2status(content_type_status))
         assignable.setBlacklistStatus(CONTEXT_CATEGORY, int2status(context_status))
 
         baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
@@ -96,13 +111,18 @@ class ManageContextualPortlets(BrowserView):
 
     def convert_legacy_portlets(self):
         utils.convert_legacy_portlets(self.context)
-        self.context.request.response.redirect(self.context.absolute_url() + '/@@manage-portlets')
+        self.request.response.redirect(self.context.absolute_url() + '/@@manage-portlets')
 
+
+@implementer(IManageDashboardPortletsView)
 class ManageDashboardPortlets(BrowserView):
-    implements(IManageDashboardPortletsView)
 
     # IManagePortletsView implementation
 
+    @property
+    def auth_token(self):
+        return createToken()
+
     @property
     def macros(self):
         return self.index.macros
@@ -126,26 +146,64 @@ class ManageDashboardPortlets(BrowserView):
         category = column[USER_CATEGORY]
         mapping = category.get(userId, None)
         if mapping is None:
-            mapping = category[userId] = UserPortletAssignmentMapping(manager=manager.__name__,
-                                                                      category=USER_CATEGORY,
-                                                                      name=userId)
+            mapping = category[userId] = UserPortletAssignmentMapping(
+                manager=manager.__name__, category=USER_CATEGORY, name=userId)
         return mapping.values()
 
     def _getUserId(self):
         membership = getToolByName(aq_inner(self.context), 'portal_membership', None)
         if membership.isAnonymousUser():
-            raise Unauthorized, "Cannot get portlet assignments for anonymous through this view"
+            raise Unauthorized("Cannot get portlet assignments for anonymous "
+                               "through this view")
 
         member = membership.getAuthenticatedMember()
         memberId = member.getId()
 
         if memberId is None:
-            raise KeyError, "Cannot find user id of current user"
+            raise KeyError("Cannot find user id of current user")
 
         return memberId
 
+
+@implementer(IManageDashboardPortletsView)
+class ManageGroupDashboardPortlets(BrowserView):
+
+    @property
+    def group(self):
+        return self.request.get('key', None)
+
+    # IManagePortletsView implementation
+
+    @property
+    def macros(self):
+        return self.index.macros
+
+    @property
+    def category(self):
+        return GROUP_CATEGORY
+
+    @property
+    def key(self):
+        return self.group
+
+    def getAssignmentMappingUrl(self, manager):
+        baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
+        return '%s/++groupdashboard++%s+%s' % (baseUrl, manager.__name__, self.group)
+
+    def getAssignmentsForManager(self, manager):
+        column = getUtility(IPortletManager, name=manager.__name__)
+        category = column[GROUP_CATEGORY]
+        mapping = category.get(self.group, None)
+        if mapping is None:
+            mapping = category[self.group] = \
+                GroupDashboardPortletAssignmentMapping(manager=manager.__name__,
+                                                       category=GROUP_CATEGORY,
+                                                       name=self.group)
+        return mapping.values()
+
+
+@implementer(IManageGroupPortletsView)
 class ManageGroupPortlets(BrowserView):
-    implements(IManageGroupPortletsView)
 
     # IManagePortletsView implementation
 
@@ -186,8 +244,9 @@ class ManageGroupPortlets(BrowserView):
     def group(self):
         return self.request['key']
 
+
+@implementer(IManageContentTypePortletsView)
 class ManageContentTypePortlets(BrowserView):
-    implements(IManageContentTypePortletsView)
 
     def __init__(self, context, request):
         super(ManageContentTypePortlets, self).__init__(context, request)
@@ -218,9 +277,8 @@ class ManageContentTypePortlets(BrowserView):
         category = column[CONTENT_TYPE_CATEGORY]
         mapping = category.get(pt, None)
         if mapping is None:
-            mapping = category[pt] = PortletAssignmentMapping(manager=manager.__name__,
-                                                              category=CONTENT_TYPE_CATEGORY,
-                                                              name=pt)
+            mapping = category[pt] = PortletAssignmentMapping(
+                manager=manager.__name__, category=CONTENT_TYPE_CATEGORY, name=pt)
         return mapping.values()
 
     # View attributes
@@ -229,8 +287,9 @@ class ManageContentTypePortlets(BrowserView):
         return self.fti().Title()
 
     def portal_type_icon(self):
-        ploneview = getMultiAdapter((self.context, self.request), name=u"plone")
-        return ploneview.getIcon(self.fti())
+        plone_layout = getMultiAdapter(
+            (self.context, self.request), name=u"plone_layout")
+        return plone_layout.getIcon(self.fti())
 
     @memoize
     def fti(self):
@@ -240,6 +299,8 @@ class ManageContentTypePortlets(BrowserView):
             if fti.getId() == portal_type:
                 return fti
 
+
+@implementer(IManagePortletsView)
 class ManagePortletsViewlet(BrowserView):
     """A general base class for viewlets that want to be rendered on the
     manage portlets view. This makes it possible to have a viewlet that
@@ -248,7 +309,6 @@ class ManagePortletsViewlet(BrowserView):
     renderer is registered on IManagePortletsView, but inside a viewlet,
     the __parent__ is the viewlet, not the ultimate parent).
     """
-    implements(IManagePortletsView)
 
     def __init__(self, context, request, view, manager):
         super(ManagePortletsViewlet, self).__init__(context, request)
@@ -270,6 +330,13 @@ class ManagePortletsViewlet(BrowserView):
     def key(self):
         return self.ultimate_parent().key
 
+    def __getattribute__(self, name):
+        # Products.Five.viewlet.viewlet.SimpleViewletClass redefines __name__
+        # so a simple property or attribute does not work
+        if name == '__name__':
+            return self.ultimate_parent().__name__
+        return super(ManagePortletsViewlet, self).__getattribute__(name)
+
     def getAssignmentMappingUrl(self, manager):
         return self.ultimate_parent().getAssignmentMappingUrl(manager)
 
@@ -278,8 +345,10 @@ class ManagePortletsViewlet(BrowserView):
 
     @memoize
     def ultimate_parent(self):
+        # Walk the __parent__ chain to find the principal view
         parent = self.__parent__
-        while parent is not None and hasattr(parent, '__parent__') and parent.__parent__ is not None:
+        while (hasattr(parent, '__parent__') and
+                IBrowserView.providedBy(parent.__parent__)):
             parent = parent.__parent__
         return parent
 
@@ -287,20 +356,54 @@ class ManagePortletsViewlet(BrowserView):
     # we can use ZCML with the template attribute (hence we don't
     # put these here)
 
+
+@implementer(IManageContextualPortletsView)
 class ManageContextualPortletsViewlet(ManagePortletsViewlet):
     """A viewlet base class for viewlets that need to render on the
     manage contextual portlets screen.
     """
-    implements(IManageContextualPortletsView)
 
+
+@implementer(IManageGroupPortletsView)
 class ManageGroupPortletsViewlet(ManagePortletsViewlet):
     """A viewlet base class for viewlets that need to render on the
     manage group portlets screen.
     """
-    implements(IManageGroupPortletsView)
 
+
+@implementer(IManageContentTypePortletsView)
 class ManageContentTypePortletsViewlet(ManagePortletsViewlet):
     """A viewlet base class for viewlets that need to render on the
     manage content type portlets screen.
     """
-    implements(IManageContentTypePortletsView)
+
+
+@implementer_only(ITopbarManagePortlets)
+class TopbarManagePortlets(ManageContextualPortlets):
+
+    def __init__(self, context, request):
+        super(TopbarManagePortlets, self).__init__(context, request)
+        # Disable the left and right columns
+        self.request.set('disable_plone.leftcolumn', 1)
+        self.request.set('disable_plone.rightcolumn', 1)
+        # Initialize the manager name in case there is nothing
+        # in the traversal path
+        self.manager_name = 'plone.leftcolumn'
+
+    def publishTraverse(self, request, name):
+        """Get the portlet manager via traversal so that we can re-use
+        the portlet machinery without overriding it all here.
+        """
+        self.manager_name = name
+        return self
+
+    def render_edit_manager_portlets(self):
+        # Pay attention again
+        # Here we manually render the portlets instead of doing
+        # something like provider:${view/manager_name} in the template
+        manager_view = ManageContextualPortlets(self.context, self.request)
+        manager_view.__name__ = 'manage-portlets'
+        portlet_manager = getMultiAdapter(
+            (self.context, self.request, manager_view), name=self.manager_name)
+        portlet_manager.update()
+        return portlet_manager.render()
\ No newline at end of file
