@@ -1,4 +1,5 @@
-from zope.interface import implements
+# -*- coding: utf-8 -*-
+from zope.interface import implementer
 from zope.component import adapts, getUtility, getMultiAdapter
 
 from zope.traversing.interfaces import ITraversable
@@ -16,11 +17,12 @@ from plone.portlets.constants import CONTENT_TYPE_CATEGORY
 
 from plone.app.portlets.storage import PortletAssignmentMapping
 from plone.app.portlets.storage import UserPortletAssignmentMapping
+from plone.app.portlets.storage import GroupDashboardPortletAssignmentMapping
 
+@implementer(ITraversable)
 class ContextPortletNamespace(object):
     """Used to traverse to a contextual portlet assignable
     """
-    implements(ITraversable)
     adapts(ILocalPortletAssignable, IHTTPRequest)
 
     def __init__(self, context, request=None):
@@ -32,11 +34,11 @@ class ContextPortletNamespace(object):
         manager = getMultiAdapter((self.context, column,), IPortletAssignmentMapping)
         return manager
 
+@implementer(ITraversable)
 class DashboardNamespace(object):
     """Used to traverse to a portlet assignable for the current user for
     the dashboard.
     """
-    implements(ITraversable)
     adapts(ISiteRoot, IHTTPRequest)
 
     def __init__(self, context, request=None):
@@ -49,9 +51,8 @@ class DashboardNamespace(object):
         category = column[USER_CATEGORY]
         manager = category.get(user, None)
         if manager is None:
-            manager = category[user] = UserPortletAssignmentMapping(manager=col,
-                                                                    category=USER_CATEGORY,
-                                                                    name=user)
+            manager = category[user] = UserPortletAssignmentMapping(
+                manager=col, category=USER_CATEGORY, name=user)
 
         # XXX: For graceful migration
         if not getattr(manager, '__manager__', None):
@@ -63,10 +64,32 @@ class DashboardNamespace(object):
 
         return manager
 
+@implementer(ITraversable)
+class GroupDashboardNamespace(object):
+    """Used to traverse to a portlet assignable for a group for the dashboard
+    """
+    adapts(ISiteRoot, IHTTPRequest)
+
+    def __init__(self, context, request=None):
+        self.context = context
+        self.request = request
+
+    def traverse(self, name, ignore):
+        col, group = name.split('+')
+        column = getUtility(IPortletManager, name=col)
+        category = column[GROUP_CATEGORY]
+        manager = category.get(group, None)
+        if manager is None:
+            manager = category[group] = \
+                GroupDashboardPortletAssignmentMapping(manager=col,
+                                                       category=GROUP_CATEGORY,
+                                                       name=group)
+        return manager
+
+@implementer(ITraversable)
 class GroupPortletNamespace(object):
     """Used to traverse to a group portlet assignable
     """
-    implements(ITraversable)
     adapts(ISiteRoot, IHTTPRequest)
 
     def __init__(self, context, request=None):
@@ -93,10 +116,10 @@ class GroupPortletNamespace(object):
 
         return manager
 
+@implementer(ITraversable)
 class ContentTypePortletNamespace(object):
     """Used to traverse to a content type portlet assignable
     """
-    implements(ITraversable)
     adapts(ISiteRoot, IHTTPRequest)
 
     def __init__(self, context, request=None):
@@ -109,9 +132,8 @@ class ContentTypePortletNamespace(object):
         category = column[CONTENT_TYPE_CATEGORY]
         manager = category.get(pt, None)
         if manager is None:
-            manager = category[pt] = PortletAssignmentMapping(manager=col,
-                                                              category=CONTENT_TYPE_CATEGORY,
-                                                              name=pt)
+            manager = category[pt] = PortletAssignmentMapping(
+                manager=col, category=CONTENT_TYPE_CATEGORY, name=pt)
 
         # XXX: For graceful migration
         if not getattr(manager, '__manager__', None):
