@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from zope.interface import Interface
 
 from zope import schema
@@ -6,7 +7,8 @@ from zope.configuration import fields as configuration_fields
 from zope.publisher.interfaces.browser import IDefaultBrowserLayer, IBrowserView
 from plone.portlets.interfaces import IPortletManager
 
-from Products.CMFPlone import PloneMessageFactory as _
+from plone.app.portlets import PloneMessageFactory as _
+
 
 class IPortletDirective(Interface):
     """Directive which registers a new portlet type.
@@ -41,7 +43,7 @@ class IPortletDirective(Interface):
         title=_(u"Edit permission"),
         description=_(u"Permission used for editing the portlet assignment."),
         required=False,
-        default=u"cmf.ManagePortal"
+        default=u"plone.app.portlets.ManageOwnPortlets"
         )
 
     renderer = configuration_fields.GlobalObject(
@@ -62,6 +64,7 @@ class IPortletDirective(Interface):
         required=False
         )
 
+
 class IPortletRendererDirective(Interface):
     """Register a portlet renderer, i.e. a different view of a portlet
     """
