@@ -1,28 +1,31 @@
-from zope.interface import implements
-
-from plone.portlets.interfaces import IPortletDataProvider
+# -*- coding: utf-8 -*-
+from plone.app.portlets import PloneMessageFactory as _
 from plone.app.portlets.portlets import base
-
-from zope import schema
-from zope.formlib import form
+from plone.portlets.interfaces import IPortletDataProvider
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
-from Products.CMFPlone import PloneMessageFactory as _
+from zope import schema
+from zope.interface import implementer
+
 
 class IClassicPortlet(IPortletDataProvider):
     """A portlet which can render a classic Plone portlet macro
     """
 
-    template = schema.ASCIILine(title=_(u'Template'),
-                            description=_(u'The template containing the portlet.'),
-                            required=True)
+    template = schema.ASCIILine(
+        title=_(u'Template'),
+        description=_(u'The template containing the portlet.'),
+        required=True)
 
-    macro = schema.ASCIILine(title=_(u'Macro'),
-                         description=_(u"The macro containing the portlet."),
-                         default='portlet',
-                         required=True)
+    macro = schema.ASCIILine(
+        title=_(u'Macro'),
+        description=_(u"The macro containing the portlet. "
+                      u"Leave blank if there is no macro."),
+        default='portlet',
+        required=False)
 
+
+@implementer(IClassicPortlet)
 class Assignment(base.Assignment):
-    implements(IClassicPortlet)
 
     def __init__(self, template='', macro=''):
         self.template = template
@@ -32,10 +35,12 @@ class Assignment(base.Assignment):
     def title(self):
         return self.template
 
+
 class Renderer(base.Renderer):
 
     def __init__(self, context, request, view, manager, data):
         self.context = context
+        self.request = request
         self.data = data
 
     render = ViewPageTemplateFile('classic.pt')
@@ -49,16 +54,20 @@ class Renderer(base.Renderer):
             expr += '/macros/%s' % self.data.macro
         return expr
 
+
 class AddForm(base.AddForm):
-    form_fields = form.Fields(IClassicPortlet)
+    schema = IClassicPortlet
     label = _(u"Add Classic Portlet")
-    description = _(u"A classic portlet allows you to use legacy portlet templates.")
+    description = _(u"A classic portlet allows you to use legacy portlet "
+                    u"templates.")
 
     def create(self, data):
         return Assignment(template=data.get('template', ''),
                           macro=data.get('macro', ''))
 
+
 class EditForm(base.EditForm):
-    form_fields = form.Fields(IClassicPortlet)
+    schema = IClassicPortlet
     label = _(u"Edit Classic Portlet")
-    description = _(u"A classic portlet allows you to use legacy portlet templates.")
+    description = _(u"A classic portlet allows you to use legacy portlet "
+                    u"templates.")
