@@ -1,33 +1,44 @@
-from zope.interface import implements, Interface
-from zope.component import adapts, getMultiAdapter, queryUtility
-
-from plone.i18n.normalizer.interfaces import IIDNormalizer
-from plone.portlets.interfaces import IPortletDataProvider
-from plone.app.portlets.portlets import base
-
-from zope import schema
-from zope.formlib import form
-
-from plone.memoize.instance import memoize
-
-from Acquisition import aq_inner, aq_base, aq_parent
-from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
-from Products.CMFCore.utils import getToolByName
-
-from Products.CMFPlone.interfaces import INonStructuralFolder, IBrowserDefault
-from Products.CMFPlone import utils
-from Products.CMFPlone import PloneMessageFactory as _
-
-from plone.app.layout.navigation.interfaces import INavtreeStrategy
+# -*- coding: utf-8 -*-
+from Acquisition import aq_base
+from Acquisition import aq_inner
+from Acquisition import aq_parent
+from ComputedAttribute import ComputedAttribute
 from plone.app.layout.navigation.interfaces import INavigationQueryBuilder
-
-from plone.app.layout.navigation.root import getNavigationRoot
+from plone.app.layout.navigation.interfaces import INavigationRoot
+from plone.app.layout.navigation.interfaces import INavtreeStrategy
 from plone.app.layout.navigation.navtree import buildFolderTree
-
-from plone.app.vocabularies.catalog import SearchableTextSourceBinder
-from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget
-
+from plone.app.layout.navigation.root import getNavigationRoot
+from plone.app.layout.navigation.root import getNavigationRootObject
+from plone.app.portlets import PloneMessageFactory as _
+from plone.app.portlets.portlets import base
+from plone.app.uuid.utils import uuidToObject
+from plone.app.vocabularies.catalog import CatalogSource
+from plone.i18n.normalizer.interfaces import IIDNormalizer
+from plone.memoize.instance import memoize
+from plone.portlets.interfaces import IPortletDataProvider
+from plone.registry.interfaces import IRegistry
+from Products.CMFCore.interfaces import ISiteRoot
+from Products.CMFCore.utils import getToolByName
+from Products.CMFDynamicViewFTI.interfaces import IBrowserDefault
+from Products.CMFPlone import utils
 from Products.CMFPlone.browser.navtree import SitemapNavtreeStrategy
+from Products.CMFPlone.defaultpage import is_default_page
+from Products.CMFPlone.interfaces import INavigationSchema
+from Products.CMFPlone.interfaces import INonStructuralFolder
+from Products.CMFPlone.interfaces import ISiteSchema
+from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
+from Products.MimetypesRegistry.MimeTypeItem import guess_icon_path
+from zExceptions import NotFound
+from zope import schema
+from zope.component import adapts
+from zope.component import getMultiAdapter
+from zope.component import getUtility
+from zope.component import queryUtility
+from zope.interface import implementer
+from zope.interface import Interface
+
+import os
+
 
 class INavigationPortlet(IPortletDataProvider):
     """A portlet which can render the navigation tree
@@ -36,20 +47,19 @@ class INavigationPortlet(IPortletDataProvider):
     name = schema.TextLine(
             title=_(u"label_navigation_title", default=u"Title"),
             description=_(u"help_navigation_title",
-                          default=u"The title of the navigation tree. Leave "
-                                   "blank for the default, translated title."),
+                          default=u"The title of the navigation tree."),
             default=u"",
             required=False)
 
-    root = schema.Choice(
+    root_uid = schema.Choice(
             title=_(u"label_navigation_root_path", default=u"Root node"),
             description=_(u'help_navigation_root',
                           default=u"You may search for and choose a folder "
                                     "to act as the root of the navigation tree. "
                                     "Leave blank to use the Plone site root."),
             required=False,
-            source=SearchableTextSourceBinder({'is_folderish' : True},
-                                              default_query='path:'))
+            source=CatalogSource(is_folderish=True),
+            )
 
     includeTop = schema.Bool(
             title=_(u"label_include_top_node", default=u"Include top node"),
@@ -95,49 +105,156 @@ class INavigationPortlet(IPortletDataProvider):
             default=0,
             required=False)
 
+    no_icons = schema.Bool(
+        title=_(u"Suppress Icons"),
+        description=_(
+            u"If enabled, the portlet will not show document type icons."),
+        required=True,
+        default=False)
+
+    thumb_scale = schema.TextLine(
+        title=_(u"Override thumb scale"),
+        description=_(
+            u"Enter a valid scale name"
+            u" (see 'Image Handling' control panel) to override"
+            u" (e.g. icon, tile, thumb, mini, preview, ... )."
+            u" Leave empty to use default (see 'Site' control panel)."
+        ),
+        required=False,
+        default=u'')
+
+    no_thumbs = schema.Bool(
+        title=_(u"Suppress thumbs"),
+        description=_(
+            u"If enabled, the portlet will not show thumbs."),
+        required=True,
+        default=False)
+
+
+@implementer(INavigationPortlet)
 class Assignment(base.Assignment):
-    implements(INavigationPortlet)
 
-    title = _(u'Navigation')
-
-    name = u""
+    name = ""
     root = None
+    root_uid = None
     currentFolderOnly = False
     includeTop = False
     topLevel = 1
     bottomLevel = 0
+    no_icons = False
+    thumb_scale = None
+    no_thumbs = False
 
-    def __init__(self, name=u"", root=None, currentFolderOnly=False, includeTop=False, topLevel=1, bottomLevel=0):
+    def __init__(
+            self,
+            name="",
+            root_uid=None,
+            currentFolderOnly=False,
+            includeTop=False,
+            topLevel=1,
+            bottomLevel=0,
+            no_icons=False,
+            thumb_scale=None,
+            no_thumbs=False
+    ):
         self.name = name
-        self.root = root
+        self.root_uid = root_uid
         self.currentFolderOnly = currentFolderOnly
         self.includeTop = includeTop
         self.topLevel = topLevel
         self.bottomLevel = bottomLevel
+        self.no_icons = no_icons
+        self.thumb_scale = thumb_scale
+        self.no_thumbs = no_thumbs
+
+    @property
+    def title(self):
+        """
+        Display the name in portlet mngmt interface
+        """
+        if self.name:
+            return self.name
+        return _(u'Navigation')
+
+    def _root(self):
+        # This is only called if the instance doesn't have a root_uid
+        # attribute, which is probably because it has an old 'root'
+        # attribute that needs to be converted.
+        root = self.root
+        if not root:
+            return None
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        navroot = getNavigationRootObject(self, portal)
+        try:
+            root = navroot.unrestrictedTraverse(root.lstrip('/'))
+        except (AttributeError, KeyError, TypeError, NotFound):
+            return
+        return root.UID()
+    root_uid = ComputedAttribute(_root, 1)
+
 
 class Renderer(base.Renderer):
 
     def __init__(self, context, request, view, manager, data):
         base.Renderer.__init__(self, context, request, view, manager, data)
 
-        self.properties = getToolByName(context, 'portal_properties').navtree_properties
         self.urltool = getToolByName(context, 'portal_url')
 
     def title(self):
-        return self.data.name or self.properties.name
+        return self.data.name or self.data.title
+
+    def hasName(self):
+        return self.data.name
 
     @property
     def available(self):
+        rootpath = self.getNavRootPath()
+        if rootpath is None:
+            return False
+
+        if self.data.bottomLevel < 0:
+            return True
+
         tree = self.getNavTree()
-        root = self.getNavRoot()
-        return (root is not None and len(tree['children']) > 0)
+        return len(tree['children']) > 0
 
     def include_top(self):
-        return getattr(self.data, 'includeTop', self.properties.includeTop)
+        return getattr(self.data, 'includeTop', True)
 
     def navigation_root(self):
         return self.getNavRoot()
 
+    def heading_link_target(self):
+        """
+        Get the href target where clicking the portlet header will take you.
+
+        If this is a customized portlet with a custom root item set,
+        we probably want to take the user to the custom root item instead
+        of the sitemap of the navigation root.
+
+        Plone does not have subsection sitemaps so there is no point of
+        displaying /sitemap links for anything besides nav root.
+        """
+
+        if not self.data.root_uid and not self.data.currentFolderOnly:
+            # No particular root item assigned -> should get link to the
+            # navigation root sitemap of the current context acquisition chain
+            portal_state = getMultiAdapter((self.context, self.request), name="plone_portal_state")
+            return portal_state.navigation_root_url() + "/sitemap"
+
+        nav_root = self.getNavRoot()
+
+        # Root content item gone away or similar issue
+        if not nav_root:
+            return None
+
+        if INavigationRoot.providedBy(nav_root) or ISiteRoot.providedBy(nav_root):
+            # For top level folders go to the sitemap
+            return nav_root.absolute_url() + "/sitemap"
+        else:
+            # Go to the item /view we have chosen as root item
+            return nav_root.absolute_url()
+
     def root_type_name(self):
         root = self.getNavRoot()
         return queryUtility(IIDNormalizer).normalize(root.portal_type)
@@ -145,17 +262,12 @@ class Renderer(base.Renderer):
     def root_item_class(self):
         context = aq_inner(self.context)
         root = self.getNavRoot()
+        container = aq_parent(context)
         if (aq_base(root) is aq_base(context) or
-                (aq_base(root) is aq_base(aq_parent(aq_inner(context))) and
-                utils.isDefaultPage(context, self.request, context))):
+                (aq_base(root) is aq_base(container) and
+                is_default_page(container, context))):
             return 'navTreeCurrentItem'
-        else:
-            return ''
-
-    def root_icon(self):
-        ploneview = getMultiAdapter((self.context, self.request), name=u'plone')
-        icon = ploneview.getIcon(self.getNavRoot())
-        return icon.url
+        return ''
 
     def root_is_portal(self):
         root = self.getNavRoot()
@@ -164,78 +276,126 @@ class Renderer(base.Renderer):
     def createNavTree(self):
         data = self.getNavTree()
 
-        bottomLevel = self.data.bottomLevel or self.properties.getProperty('bottomLevel', 0)
+        bottomLevel = self.data.bottomLevel or 0
 
-        return self.recurse(children=data.get('children', []), level=1, bottomLevel=bottomLevel)
+        if bottomLevel < 0:
+            # Special case where navigation tree depth is negative
+            # meaning that the admin does not want the listing to be displayed
+            return self.recurse([], level=1, bottomLevel=bottomLevel)
+        else:
+            return self.recurse(children=data.get('children', []), level=1, bottomLevel=bottomLevel)
 
     # Cached lookups
 
     @memoize
-    def getNavRoot(self, _marker=[]):
+    def getNavRootPath(self):
+        return getRootPath(self.context,
+                           self.data.currentFolderOnly,
+                           self.data.topLevel,
+                           self.data.root_uid)
+
+    @memoize
+    def getNavRoot(self, _marker=None):
+        if _marker is None:
+            _marker = []
         portal = self.urltool.getPortalObject()
-
-        currentFolderOnly = self.data.currentFolderOnly or self.properties.getProperty('currentFolderOnlyInNavtree', False)
-        topLevel = self.data.topLevel or self.properties.getProperty('topLevel', 0)
-
-        rootPath = getRootPath(self.context, currentFolderOnly, topLevel, str(self.data.root))
-
+        rootPath = self.getNavRootPath()
         if rootPath is None:
-            return rootPath
+            return None
 
         if rootPath == self.urltool.getPortalPath():
             return portal
         else:
             try:
                 return portal.unrestrictedTraverse(rootPath)
-            except (AttributeError, KeyError,):
+            except (AttributeError, KeyError, TypeError, NotFound):
+                # TypeError: object is unsubscribtable might be
+                # risen in some cases
                 return portal
 
     @memoize
-    def getNavTree(self, _marker=[]):
+    def getNavTree(self, _marker=None):
+        if _marker is None:
+            _marker = []
         context = aq_inner(self.context)
-
-        # Special case - if the root is supposed to be pruned, we need to
-        # abort here
-
         queryBuilder = getMultiAdapter((context, self.data), INavigationQueryBuilder)
         strategy = getMultiAdapter((context, self.data), INavtreeStrategy)
 
         return buildFolderTree(context, obj=context, query=queryBuilder(), strategy=strategy)
 
+
+    _template = ViewPageTemplateFile('navigation.pt')
+    recurse = ViewPageTemplateFile('navigation_recurse.pt')
+
+    @memoize
+    def thumb_scale(self):
+        """Use override value or read thumb_scale from registry.
+        Image sizes must fit to value in allowed image sizes.
+        None will suppress thumb.
+        """
+        if getattr(self.data, 'no_thumbs', False):
+            # Individual setting overrides
+            return None
+        thsize = getattr(self.data, 'thumb_scale', None)
+        if thsize:
+            return thsize
+        registry = getUtility(IRegistry)
+        settings = registry.forInterface(
+            ISiteSchema, prefix="plone", check=False)
+        if settings.no_thumbs_portlet:
+            return 'none'
+        thumb_scale_portlet = settings.thumb_scale_portlet
+        return thumb_scale_portlet
+
+    def getMimeTypeIcon(self, node):
+        try:
+            if not node['normalized_portal_type'] == 'file':
+                return None
+            fileo = node['item'].getObject().file
+            portal_url = getNavigationRoot(self.context)
+            mtt = getToolByName(self.context, 'mimetypes_registry')
+            if fileo.contentType:
+                ctype = mtt.lookup(fileo.contentType)
+                if not ctype:
+                    return None
+                return os.path.join(
+                    portal_url,
+                    guess_icon_path(ctype[0])
+                )
+        except AttributeError:
+            return None
+        return None
+
     def update(self):
         pass
 
     def render(self):
         return self._template()
 
-    _template = ViewPageTemplateFile('navigation.pt')
-    recurse = ViewPageTemplateFile('navigation_recurse.pt')
 
 class AddForm(base.AddForm):
-    form_fields = form.Fields(INavigationPortlet)
-    form_fields['root'].custom_widget = UberSelectionWidget
+    schema = INavigationPortlet
     label = _(u"Add Navigation Portlet")
-    description = _(u"This portlet display a navigation tree.")
+    description = _(u"This portlet displays a navigation tree.")
 
     def create(self, data):
-        return Assignment(name=data.get('name', u""),
-                          root=data.get('root', u""),
+        return Assignment(name=data.get('name', ""),
+                          root_uid=data.get('root_uid', ""),
                           currentFolderOnly=data.get('currentFolderOnly', False),
                           includeTop=data.get('includeTop', False),
                           topLevel=data.get('topLevel', 0),
                           bottomLevel=data.get('bottomLevel', 0))
 
 class EditForm(base.EditForm):
-    form_fields = form.Fields(INavigationPortlet)
-    form_fields['root'].custom_widget = UberSelectionWidget
+    schema = INavigationPortlet
     label = _(u"Edit Navigation Portlet")
-    description = _(u"This portlet display a navigation tree.")
+    description = _(u"This portlet displays a navigation tree.")
 
+@implementer(INavigationQueryBuilder)
 class QueryBuilder(object):
-    """Build a navtree query based on the settings in navtree_properties
+    """Build a navtree query based on the settings in INavigationSchema
     and those set on the portlet.
     """
-    implements(INavigationQueryBuilder)
     adapts(Interface, INavigationPortlet)
 
     def __init__(self, context, portlet):
@@ -245,8 +405,6 @@ class QueryBuilder(object):
         portal_properties = getToolByName(context, 'portal_properties')
         navtree_properties = getattr(portal_properties, 'navtree_properties')
 
-        portal_url = getToolByName(context, 'portal_url')
-
         # Acquire a custom nav query if available
         customQuery = getattr(context, 'getCustomNavQuery', None)
         if customQuery is not None and utils.safe_callable(customQuery):
@@ -255,22 +413,25 @@ class QueryBuilder(object):
             query = {}
 
         # Construct the path query
-
-        rootPath = getNavigationRoot(context, relativeRoot=portlet.root)
+        root = uuidToObject(portlet.root_uid)
+        if root is not None:
+            rootPath = '/'.join(root.getPhysicalPath())
+        else:
+            rootPath = getNavigationRoot(context)
         currentPath = '/'.join(context.getPhysicalPath())
 
         # If we are above the navigation root, a navtree query would return
         # nothing (since we explicitly start from the root always). Hence,
         # use a regular depth-1 query in this case.
 
-        if currentPath!=rootPath and not currentPath.startswith(rootPath+'/'):
-            query['path'] = {'query' : rootPath, 'depth' : 1}
+        if currentPath != rootPath and not currentPath.startswith(rootPath + '/'):
+            query['path'] = {'query': rootPath, 'depth': 1}
         else:
-            query['path'] = {'query' : currentPath, 'navtree' : 1}
+            query['path'] = {'query': currentPath, 'navtree': 1}
 
-        topLevel = portlet.topLevel or navtree_properties.getProperty('topLevel', 0)
+        topLevel = portlet.topLevel
         if topLevel and topLevel > 0:
-             query['path']['navtree_start'] = topLevel + 1
+            query['path']['navtree_start'] = topLevel + 1
 
         # XXX: It'd make sense to use 'depth' for bottomLevel, but it doesn't
         # seem to work with EPI.
@@ -287,31 +448,36 @@ class QueryBuilder(object):
                 query['sort_order'] = sortOrder
 
         # Filter on workflow states, if enabled
-        if navtree_properties.getProperty('enable_wf_state_filtering', False):
-            query['review_state'] = navtree_properties.getProperty('wf_states_to_show', ())
+        registry = getUtility(IRegistry)
+        navigation_settings = registry.forInterface(
+            INavigationSchema,
+            prefix="plone"
+        )
+        if navigation_settings.filter_on_workflow:
+            query['review_state'] = navigation_settings.workflow_states_to_show
 
         self.query = query
 
     def __call__(self):
         return self.query
 
+
+@implementer(INavtreeStrategy)
 class NavtreeStrategy(SitemapNavtreeStrategy):
     """The navtree strategy used for the default navigation portlet
     """
-    implements(INavtreeStrategy)
     adapts(Interface, INavigationPortlet)
 
     def __init__(self, context, portlet):
         SitemapNavtreeStrategy.__init__(self, context, portlet)
-        portal_properties = getToolByName(context, 'portal_properties')
-        navtree_properties = getattr(portal_properties, 'navtree_properties')
 
         # XXX: We can't do this with a 'depth' query to EPI...
-        self.bottomLevel = portlet.bottomLevel or navtree_properties.getProperty('bottomLevel', 0)
+        self.bottomLevel = portlet.bottomLevel or 0
 
-        currentFolderOnly = portlet.currentFolderOnly or navtree_properties.getProperty('currentFolderOnlyInNavtree', False)
-        topLevel = portlet.topLevel or navtree_properties.getProperty('topLevel', 0)
-        self.rootPath = getRootPath(context, currentFolderOnly, topLevel, portlet.root)
+        self.rootPath = getRootPath(context,
+                                    portlet.currentFolderOnly,
+                                    portlet.topLevel,
+                                    portlet.root_uid)
 
     def subtreeFilter(self, node):
         sitemapDecision = SitemapNavtreeStrategy.subtreeFilter(self, node)
@@ -323,12 +489,14 @@ class NavtreeStrategy(SitemapNavtreeStrategy):
         else:
             return True
 
+
 def getRootPath(context, currentFolderOnly, topLevel, root):
     """Helper function to calculate the real root path
     """
     context = aq_inner(context)
     if currentFolderOnly:
-        folderish = getattr(aq_base(context), 'isPrincipiaFolderish', False) and not INonStructuralFolder.providedBy(context)
+        folderish = getattr(aq_base(context), 'isPrincipiaFolderish', False) and \
+                    not INonStructuralFolder.providedBy(context)
         parent = aq_parent(context)
 
         is_default_page = False
@@ -341,14 +509,18 @@ def getRootPath(context, currentFolderOnly, topLevel, root):
         else:
             return '/'.join(context.getPhysicalPath())
 
-    rootPath = getNavigationRoot(context, relativeRoot=root)
+    root = uuidToObject(root)
+    if root is not None:
+        rootPath = '/'.join(root.getPhysicalPath())
+    else:
+        rootPath = getNavigationRoot(context)
 
     # Adjust for topLevel
     if topLevel > 0:
         contextPath = '/'.join(context.getPhysicalPath())
         if not contextPath.startswith(rootPath):
             return None
-        contextSubPathElements = contextPath[len(rootPath)+1:]
+        contextSubPathElements = contextPath[len(rootPath) + 1:]
         if contextSubPathElements:
             contextSubPathElements = contextSubPathElements.split('/')
             if len(contextSubPathElements) < topLevel:
