@@ -1,35 +1,32 @@
+# -*- coding: utf-8 -*-
+from plone.app.layout.navigation.root import getNavigationRoot
+from plone.app.portlets import PloneMessageFactory as _
+from plone.app.portlets.portlets import base
+from plone.portlets.interfaces import IPortletDataProvider
+from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from zope import schema
 from zope.component import getMultiAdapter
-from zope.formlib import form
-from zope.interface import implements
+from zope.interface import implementer
 
-from plone.app.portlets.portlets import base
-from plone.memoize.instance import memoize
-from plone.portlets.interfaces import IPortletDataProvider
-
-from Acquisition import aq_inner
-from DateTime.DateTime import DateTime
-from Products.CMFCore.utils import getToolByName
-from Products.CMFPlone import PloneMessageFactory as _
-from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 
 class ISearchPortlet(IPortletDataProvider):
     """ A portlet displaying a (live) search box
     """
 
     enableLivesearch = schema.Bool(
-            title = _(u"Enable LiveSearch"),
-            description = _(u"Enables the LiveSearch feature, which shows "
-                             "live results if the browser supports "
-                             "JavaScript."),
-            default = True,
-            required = False)
+        title=_(u"Enable LiveSearch"),
+        description=_(u"Enables the LiveSearch feature, which shows "
+                      u"live results if the browser supports "
+                      u"JavaScript."),
+        default=True,
+        required=False)
 
+
+@implementer(ISearchPortlet)
 class Assignment(base.Assignment):
-    implements(ISearchPortlet)
 
     def __init__(self, enableLivesearch=True):
-        self.enableLivesearch=enableLivesearch
+        self.enableLivesearch = enableLivesearch
 
     @property
     def title(self):
@@ -39,25 +36,28 @@ class Assignment(base.Assignment):
 class Renderer(base.Renderer):
 
     render = ViewPageTemplateFile('search.pt')
+    action = '@@search'
+    livesearch_action = 'livesearch_reply'
 
     def __init__(self, context, request, view, manager, data):
         base.Renderer.__init__(self, context, request, view, manager, data)
 
-        portal_state = getMultiAdapter((context, request), name=u'plone_portal_state')
+        portal_state = getMultiAdapter(
+            (context, request), name='plone_portal_state')
         self.navigation_root_url = portal_state.navigation_root_url()
 
     def enable_livesearch(self):
         return self.data.enableLivesearch
 
-    def search_form(self):
-        return '%s/search_form' % self.navigation_root_url
-
     def search_action(self):
-        return '%s/search' % self.navigation_root_url
+        return '{0}/{1}'.format(self.navigation_root_url, self.action)
+
+    def navigation_root_url(self):
+        return getNavigationRoot(self.context)
 
 
 class AddForm(base.AddForm):
-    form_fields = form.Fields(ISearchPortlet)
+    schema = ISearchPortlet
     label = _(u"Add Search Portlet")
     description = _(u"This portlet shows a search box.")
 
@@ -66,6 +66,6 @@ class AddForm(base.AddForm):
 
 
 class EditForm(base.EditForm):
-    form_fields = form.Fields(ISearchPortlet)
+    schema = ISearchPortlet
     label = _(u"Edit Search Portlet")
     description = _(u"This portlet shows a search box.")
