@@ -1,6 +1,5 @@
+# -*- coding: utf-8 -*-
 from zope.component import getUtility, getMultiAdapter
-from zope.app.component.hooks import setHooks, setSite
-
 from Products.GenericSetup.utils import _getDottedName
 
 from plone.portlets.interfaces import IPortletType
@@ -14,30 +13,31 @@ from plone.app.portlets.portlets import classic
 from plone.app.portlets.storage import PortletAssignmentMapping
 
 from plone.app.portlets.tests.base import PortletsTestCase
+from plone.app.testing import TEST_USER_ID
+from plone.app.testing import setRoles
+
 
 class TestPortlet(PortletsTestCase):
 
     def afterSetUp(self):
-        setHooks()
-        setSite(self.portal)
-        self.setRoles(('Manager',))
+        setRoles(self.portal, TEST_USER_ID, ['Manager'])
 
     def testPortletTypeRegistered(self):
         portlet = getUtility(IPortletType, name='portlets.Classic')
-        self.assertEquals(portlet.addview, 'portlets.Classic')
+        self.assertEqual(portlet.addview, 'portlets.Classic')
 
     def testRegisteredInterfaces(self):
         portlet = getUtility(IPortletType, name='portlets.Classic')
         registered_interfaces = [_getDottedName(i) for i in portlet.for_]
         registered_interfaces.sort()
-        self.assertEquals(['plone.app.portlets.interfaces.IColumn',
+        self.assertEqual(['plone.app.portlets.interfaces.IColumn',
           'plone.app.portlets.interfaces.IDashboard'],
           registered_interfaces)
 
     def testInterfaces(self):
         portlet = classic.Assignment(template='portlet_recent', macro='portlet')
-        self.failUnless(IPortletAssignment.providedBy(portlet))
-        self.failUnless(IPortletDataProvider.providedBy(portlet.data))
+        self.assertTrue(IPortletAssignment.providedBy(portlet))
+        self.assertTrue(IPortletDataProvider.providedBy(portlet.data))
 
     def testInvokeAddview(self):
         portlet = getUtility(IPortletType, name='portlets.Classic')
@@ -48,8 +48,8 @@ class TestPortlet(PortletsTestCase):
 
         addview.createAndAdd(data={})
 
-        self.assertEquals(len(mapping), 1)
-        self.failUnless(isinstance(mapping.values()[0], classic.Assignment))
+        self.assertEqual(len(mapping), 1)
+        self.assertTrue(isinstance(mapping.values()[0], classic.Assignment))
 
     def testInvokeEditView(self):
         mapping = PortletAssignmentMapping()
@@ -57,7 +57,7 @@ class TestPortlet(PortletsTestCase):
 
         mapping['foo'] = classic.Assignment(template='portlet_recent', macro='portlet')
         editview = getMultiAdapter((mapping['foo'], request), name='edit')
-        self.failUnless(isinstance(editview, classic.EditForm))
+        self.assertTrue(isinstance(editview, classic.EditForm))
 
     def testRenderer(self):
         context = self.folder
@@ -67,14 +67,11 @@ class TestPortlet(PortletsTestCase):
         assignment = classic.Assignment(template='portlet_recent', macro='portlet')
 
         renderer = getMultiAdapter((context, request, view, manager, assignment), IPortletRenderer)
-        self.failUnless(isinstance(renderer, classic.Renderer))
+        self.assertTrue(isinstance(renderer, classic.Renderer))
+
 
 class TestRenderer(PortletsTestCase):
 
-    def afterSetUp(self):
-        setHooks()
-        setSite(self.portal)
-
     def renderer(self, context=None, request=None, view=None, manager=None, assignment=None):
         context = context or self.folder
         request = request or self.folder.REQUEST
@@ -86,15 +83,20 @@ class TestRenderer(PortletsTestCase):
 
     def testUseMacro(self):
         r = self.renderer(assignment=classic.Assignment(template='portlet_recent', macro='portlet'))
-        self.assertEquals(True, r.use_macro())
+        self.assertEqual(True, r.use_macro())
         r = self.renderer(assignment=classic.Assignment(template='portlet_recent', macro=None))
-        self.assertEquals(False, r.use_macro())
+        self.assertEqual(False, r.use_macro())
 
     def testPathExpression(self):
         r = self.renderer(assignment=classic.Assignment(template='portlet_recent', macro='portlet'))
-        self.assertEquals('context/portlet_recent/macros/portlet', r.path_expression())
+        self.assertEqual('context/portlet_recent/macros/portlet', r.path_expression())
         r = self.renderer(assignment=classic.Assignment(template='portlet_recent', macro=None))
-        self.assertEquals('context/portlet_recent', r.path_expression())
+        self.assertEqual('context/portlet_recent', r.path_expression())
+
+    def testRenderClassicPortlet(self):
+        r = self.renderer(assignment=classic.Assignment(template='@@view', macro='content-core'))
+        r.render()
+
 
 def test_suite():
     from unittest import TestSuite, makeSuite
