@@ -1,5 +1,6 @@
+# -*- coding: utf-8 -*-
 from zope.component import getUtility, getMultiAdapter
-from zope.app.container.interfaces import INameChooser
+from zope.container.interfaces import INameChooser
 
 from plone.portlets.interfaces import IPortletManager
 from plone.portlets.interfaces import IPortletAssignmentMapping
@@ -8,11 +9,9 @@ from plone.portlets.constants import CONTEXT_CATEGORY, USER_CATEGORY
 
 from plone.app.portlets.interfaces import IPortletPermissionChecker
 
-from plone.app.portlets.portlets import calendar
 from plone.app.portlets.portlets import classic
 from plone.app.portlets.portlets import login
 from plone.app.portlets.portlets import news
-from plone.app.portlets.portlets import events
 from plone.app.portlets.portlets import navigation
 from plone.app.portlets.portlets import recent
 from plone.app.portlets.portlets import review
@@ -23,6 +22,14 @@ from plone.app.portlets.storage import UserPortletAssignmentMapping
 from Acquisition import aq_base
 from Products.CMFCore.utils import getToolByName
 
+from plone.app.portlets import HAS_PLONE_APP_EVENT
+if HAS_PLONE_APP_EVENT:
+    from plone.app.event.portlets import portlet_calendar as calendar
+    from plone.app.event.portlets import portlet_events as events
+
+import six
+
+
 def assignment_mapping_from_key(context, manager_name, category, key, create=False):
     """Given the name of a portlet manager, the name of a category, and a
     key in that category, return the IPortletAssignmentMapping.
@@ -45,9 +52,11 @@ def assignment_mapping_from_key(context, manager_name, category, key, create=Fal
                 path = path[len(portal_path)+1:]
             while path.startswith('/'):
                 path = path[1:]
+            if six.PY2 and isinstance(path, six.text_type):
+                path = path.encode('utf-8')  # OFS.traversable cannot do unicode
             obj = portal.restrictedTraverse(path, None)
         if obj is None:
-            raise KeyError, "Cannot find object at path %s" % path
+            raise KeyError("Cannot find object at path %s" % path)
         return getMultiAdapter((obj, manager), IPortletAssignmentMapping)
     else:
         mapping = manager[category]
@@ -58,6 +67,7 @@ def assignment_mapping_from_key(context, manager_name, category, key, create=Fal
                 mapping[key] = PortletAssignmentMapping()
         return mapping[key]
 
+
 def assignment_from_key(context, manager_name, category, key, name):
     """Given the name of a portlet manager, the name of a category, a
     key in that category and the name of a particular assignment, return
@@ -67,22 +77,29 @@ def assignment_from_key(context, manager_name, category, key, name):
 
 DONT_MIGRATE = object()
 
-portletsMapping = { 'portlet_login'      : login.Assignment(),
-                    'portlet_news'       : news.Assignment(count=5),
-                    'portlet_events'     : events.Assignment(count=5),
-                    'portlet_navigation' : navigation.Assignment(),
-                    'portlet_calendar'   : calendar.Assignment(),
-                    'portlet_review'     : review.Assignment(),
-                    'portlet_recent'     : recent.Assignment(count=5),
-                    'portlet_related'    : DONT_MIGRATE,
-                    'portlet_languages'  : DONT_MIGRATE,
-                  }
 
 def convert_legacy_portlets(context):
     """Convert legacy portlets (left_slots, right_slots) in the given
     context to new-style portlets.
     """
 
+    portletsMapping = {'portlet_login': login.Assignment(),
+                       'portlet_news': news.Assignment(count=5),
+                       'portlet_navigation': navigation.Assignment(),
+                       'portlet_review': review.Assignment(),
+                       'portlet_recent': recent.Assignment(count=5),
+                       'portlet_related': DONT_MIGRATE,
+                       'portlet_languages': DONT_MIGRATE,
+                       'portlet_calendar': DONT_MIGRATE,
+                       'portlet_events': DONT_MIGRATE,
+                       }
+
+    if HAS_PLONE_APP_EVENT:
+        portletsMapping.update({
+            'portlet_calendar': calendar.Assignment(),
+            'portlet_events': events.Assignment(count=5),
+        })
+
     # Convert left_slots and right_slots to portlets
 
     left = getUtility(IPortletManager, name='plone.leftcolumn')
@@ -104,7 +121,7 @@ def convert_legacy_portlets(context):
         path = item.split('/')
         if len(path) == 4:
             newPortlet = portletsMapping.get(path[1], None)
-            if newPortlet is None and path[0] in ('context', 'here',) and path[2] == 'macros':
+            if newPortlet is None and path[0] in ('context', 'here') and path[2] == 'macros':
                 newPortlet = classic.Assignment(path[1], path[3])
             if newPortlet is not None and newPortlet is not DONT_MIGRATE:
                 leftAssignable[leftChooser.chooseName(None, newPortlet)] = newPortlet
@@ -113,7 +130,7 @@ def convert_legacy_portlets(context):
         path = item.split('/')
         if len(path) == 4:
             newPortlet = portletsMapping.get(path[1], None)
-            if newPortlet is None and path[0] in ('context', 'here',) and path[2] == 'macros':
+            if newPortlet is None and path[0] in ('context', 'here') and path[2] == 'macros':
                 newPortlet = classic.Assignment(path[1], path[3])
             if newPortlet is not None and newPortlet is not DONT_MIGRATE:
                 rightAssignable[rightChooser.chooseName(None, newPortlet)] = newPortlet
